/***********************************************************************
 *
 * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include "settings_dialog.h"

#include <QDialogButtonBox>
#include <QDir>
#include <QGridLayout>
#include <QFileDialog>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSettings>

//-----------------------------------------------------------------------------

SettingsDialog::SettingsDialog(bool show_warning, QWidget* parent)
: QDialog(parent), m_new(false), m_changed(false) {
	setWindowTitle(tr("Settings"));

	QSettings settings;

	m_dice = new QLineEdit(this);
	m_dice->setText(settings.value("Dice").toString());
	QPushButton* choose_dice = new QPushButton(tr("Choose..."), this);
	connect(choose_dice, SIGNAL(clicked()), this, SLOT(chooseDice()));

	m_words = new QLineEdit(this);
	m_words->setText(settings.value("Words").toString());
	QPushButton* choose_words = new QPushButton(tr("Choose..."), this);
	connect(choose_words, SIGNAL(clicked()), this, SLOT(chooseWords()));

	m_dictionary = new QLineEdit(this);
	m_dictionary->setText(settings.value("Dictionary").toString());

	// Creat warning message
	QLabel* warning_img = new QLabel(this);
	warning_img->setPixmap(QString(":/dialog-warning"));
	QLabel* warning_text = new QLabel(tr("<b>Note:</b> Changing these settings will start a new game."), this);

	QHBoxLayout* warning = new QHBoxLayout;
	warning->addWidget(warning_img);
	warning->addWidget(warning_text);
	warning->addStretch();

	// Create buttons
	m_buttons = new QDialogButtonBox(QDialogButtonBox::RestoreDefaults | QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, this);
	connect(m_buttons, SIGNAL(accepted()), this, SLOT(accept()));
	connect(m_buttons, SIGNAL(rejected()), this, SLOT(reject()));
	connect(m_buttons, SIGNAL(clicked(QAbstractButton*)), this, SLOT(clicked(QAbstractButton*)));

	// Lay out window
	QGridLayout* layout = new QGridLayout(this);
	layout->setColumnStretch(1, 1);
	layout->setRowStretch(5, 1);
	layout->setRowMinimumHeight(5, 12);
	layout->setColumnMinimumWidth(1, warning->sizeHint().width());

	layout->addWidget(new QLabel(tr("Dice:"), this), 0, 0, Qt::AlignRight | Qt::AlignVCenter);
	layout->addWidget(m_dice, 0, 1);
	layout->addWidget(choose_dice, 0, 2);

	layout->addWidget(new QLabel(tr("Word list:"), this), 1, 0, Qt::AlignRight | Qt::AlignVCenter);
	layout->addWidget(m_words, 1, 1);
	layout->addWidget(choose_words, 1, 2);

	layout->addLayout(warning, 2, 1);
	if (show_warning) {
		layout->setRowMinimumHeight(3, 12);
	} else {
		warning_img->hide();
		warning_text->hide();
	}

	layout->addWidget(new QLabel(tr("Dictionary:"), this), 4, 0, Qt::AlignRight | Qt::AlignVCenter);
	layout->addWidget(m_dictionary, 4, 1, 1, 2);

	layout->addWidget(m_buttons, 6, 0, 1, 3);
}

//-----------------------------------------------------------------------------

void SettingsDialog::accept() {
	QSettings settings;
	if (settings.value("Dice").toString() != m_dice->text()) {
		settings.setValue("Dice", m_dice->text());
		m_new = true;
		m_changed = true;
	}
	if (settings.value("Words").toString() != m_words->text()) {
		settings.setValue("Words", m_words->text());
		m_new = true;
		m_changed = true;
	}
	if (settings.value("Dictionary") != m_dictionary->text()) {
		settings.setValue("Dictionary", m_dictionary->text());
		m_changed = true;
	}

	QDialog::accept();
}

//-----------------------------------------------------------------------------

void SettingsDialog::clicked(QAbstractButton* button) {
	if (m_buttons->buttonRole(button) == QDialogButtonBox::ResetRole) {
		m_dice->setText(":/en_US/dice");
		m_words->setText(":/en_US/words");
		m_dictionary->setText("http://dictionary.reference.com/browse/%s");
	}
}

//-----------------------------------------------------------------------------

void SettingsDialog::chooseDice() {
	QString path = QFileDialog::getOpenFileName(this, tr("Choose Dice File"), m_dice->text());
	if (!path.isEmpty()) {
		m_dice->setText(path);
	}
}

//-----------------------------------------------------------------------------

void SettingsDialog::chooseWords() {
	QString path = QFileDialog::getOpenFileName(this, tr("Choose Word List File"), m_words->text());
	if (!path.isEmpty()) {
		m_words->setText(path);
	}
}

//-----------------------------------------------------------------------------
